@@ -1,42 +1,35 @@
 # Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
-# For details: https://github.com/PyCQA/astroid/blob/main/LICENSE
-# Copyright (c) https://github.com/PyCQA/astroid/blob/main/CONTRIBUTORS.txt
+# For details: https://github.com/pylint-dev/astroid/blob/main/LICENSE
+# Copyright (c) https://github.com/pylint-dev/astroid/blob/main/CONTRIBUTORS.txt
 
 """
-Astroid hook for the dataclasses library
+Astroid hook for the dataclasses library.
 
 Support built-in dataclasses, pydantic.dataclasses, and marshmallow_dataclass-annotated
 dataclasses. References:
 - https://docs.python.org/3/library/dataclasses.html
 - https://pydantic-docs.helpmanual.io/usage/dataclasses/
 - https://lovasoa.github.io/marshmallow_dataclass/
-
 """
 
 from __future__ import annotations
 
-import sys
 from collections.abc import Iterator
-from typing import Tuple, Union
+from typing import Literal, Union
 
-from astroid import bases, context, helpers, nodes
+from astroid import bases, context, nodes
 from astroid.builder import parse
-from astroid.const import PY39_PLUS, PY310_PLUS
+from astroid.const import PY310_PLUS, PY313_PLUS
 from astroid.exceptions import AstroidSyntaxError, InferenceError, UseInferenceDefault
 from astroid.inference_tip import inference_tip
 from astroid.manager import AstroidManager
 from astroid.typing import InferenceResult
-from astroid.util import Uninferable
-
-if sys.version_info >= (3, 8):
-    from typing import Literal
-else:
-    from typing_extensions import Literal
+from astroid.util import Uninferable, UninferableBase, safe_infer
 
 _FieldDefaultReturn = Union[
     None,
-    Tuple[Literal["default"], nodes.NodeNG],
-    Tuple[Literal["default_factory"], nodes.Call],
+    tuple[Literal["default"], nodes.NodeNG],
+    tuple[Literal["default_factory"], nodes.Call],
 ]
 
 DATACLASSES_DECORATORS = frozenset(("dataclass",))
@@ -61,7 +54,7 @@ def is_decorated_with_dataclass(
 
 
 def dataclass_transform(node: nodes.ClassDef) -> None:
-    """Rewrite a dataclass to be easily understood by pylint"""
+    """Rewrite a dataclass to be easily understood by pylint."""
     node.is_dataclass = True
 
     for assign_node in _get_dataclass_attributes(node):
@@ -115,8 +108,7 @@ def _get_dataclass_attributes(
 ) -> Iterator[nodes.AnnAssign]:
     """Yield the AnnAssign nodes of dataclass attributes for the node.
 
-    If init is True, also include InitVars, but exclude attributes from calls to
-    field where init=False.
+    If init is True, also include InitVars.
     """
     for assign_node in node.body:
         if not isinstance(assign_node, nodes.AnnAssign) or not isinstance(
@@ -124,24 +116,15 @@ def _get_dataclass_attributes(
         ):
             continue
 
-        if _is_class_var(assign_node.annotation):  # type: ignore[arg-type] # annotation is never None
+        # Annotation is never None
+        if _is_class_var(assign_node.annotation):  # type: ignore[arg-type]
             continue
 
         if _is_keyword_only_sentinel(assign_node.annotation):
             continue
 
-        if init:
-            value = assign_node.value
-            if (
-                isinstance(value, nodes.Call)
-                and _looks_like_dataclass_field_call(value, check_scope=False)
-                and any(
-                    keyword.arg == "init" and not keyword.value.bool_value()
-                    for keyword in value.keywords
-                )
-            ):
-                continue
-        elif _is_init_var(assign_node.annotation):  # type: ignore[arg-type] # annotation is never None
+        # Annotation is never None
+        if not init and _is_init_var(assign_node.annotation):  # type: ignore[arg-type]
             continue
 
         yield assign_node
@@ -178,9 +161,11 @@ def _check_generate_dataclass_init(node: nodes.ClassDef) -> bool:
 
 
 def _find_arguments_from_base_classes(
-    node: nodes.ClassDef, skippable_names: set[str]
-) -> tuple[str, str]:
-    """Iterate through all bases and add them to the list of arguments to add to the init."""
+    node: nodes.ClassDef,
+) -> tuple[
+    dict[str, tuple[str | None, str | None]], dict[str, tuple[str | None, str | None]]
+]:
+    """Iterate through all bases and get their typing and defaults."""
     pos_only_store: dict[str, tuple[str | None, str | None]] = {}
     kw_only_store: dict[str, tuple[str | None, str | None]] = {}
     # See TODO down below
@@ -196,8 +181,6 @@ def _find_arguments_from_base_classes(
 
         pos_only, kw_only = base_init.args._get_arguments_data()
         for posarg, data in pos_only.items():
-            if posarg in skippable_names:
-                continue
             # if data[1] is None:
             #     if all_have_defaults and pos_only_store:
             #         # TODO: This should return an Uninferable as this would raise
@@ -208,10 +191,15 @@ def _find_arguments_from_base_classes(
             pos_only_store[posarg] = data
 
         for kwarg, data in kw_only.items():
-            if kwarg in skippable_names:
-                continue
             kw_only_store[kwarg] = data
+    return pos_only_store, kw_only_store
 
+
+def _parse_arguments_into_strings(
+    pos_only_store: dict[str, tuple[str | None, str | None]],
+    kw_only_store: dict[str, tuple[str | None, str | None]],
+) -> tuple[str, str]:
+    """Parse positional and keyword arguments into strings for an __init__ method."""
     pos_only, kw_only = "", ""
     for pos_arg, data in pos_only_store.items():
         pos_only += pos_arg
@@ -231,17 +219,63 @@ def _find_arguments_from_base_classes(
     return pos_only, kw_only
 
 
-def _generate_dataclass_init(
+def _get_previous_field_default(node: nodes.ClassDef, name: str) -> nodes.NodeNG | None:
+    """Get the default value of a previously defined field."""
+    for base in reversed(node.mro()):
+        if not base.is_dataclass:
+            continue
+        if name in base.locals:
+            for assign in base.locals[name]:
+                if (
+                    isinstance(assign.parent, nodes.AnnAssign)
+                    and assign.parent.value
+                    and isinstance(assign.parent.value, nodes.Call)
+                    and _looks_like_dataclass_field_call(assign.parent.value)
+                ):
+                    default = _get_field_default(assign.parent.value)
+                    if default:
+                        return default[1]
+    return None
+
+
+def _generate_dataclass_init(  # pylint: disable=too-many-locals
     node: nodes.ClassDef, assigns: list[nodes.AnnAssign], kw_only_decorated: bool
 ) -> str:
     """Return an init method for a dataclass given the targets."""
     params: list[str] = []
+    kw_only_params: list[str] = []
     assignments: list[str] = []
-    assign_names: list[str] = []
+
+    prev_pos_only_store, prev_kw_only_store = _find_arguments_from_base_classes(node)
 
     for assign in assigns:
         name, annotation, value = assign.target.name, assign.annotation, assign.value
-        assign_names.append(name)
+
+        # Check whether this assign is overriden by a property assignment
+        property_node: nodes.FunctionDef | None = None
+        for additional_assign in node.locals[name]:
+            if not isinstance(additional_assign, nodes.FunctionDef):
+                continue
+            if not additional_assign.decorators:
+                continue
+            if "builtins.property" in additional_assign.decoratornames():
+                property_node = additional_assign
+                break
+
+        is_field = isinstance(value, nodes.Call) and _looks_like_dataclass_field_call(
+            value, check_scope=False
+        )
+
+        if is_field:
+            # Skip any fields that have `init=False`
+            if any(
+                keyword.arg == "init" and not keyword.value.bool_value()
+                for keyword in value.keywords  # type: ignore[union-attr] # value is never None
+            ):
+                # Also remove the name from the previous arguments to be inserted later
+                prev_pos_only_store.pop(name, None)
+                prev_kw_only_store.pop(name, None)
+                continue
 
         if _is_init_var(annotation):  # type: ignore[arg-type] # annotation is never None
             init_var = True
@@ -255,53 +289,97 @@ def _generate_dataclass_init(
             init_var = False
             assignment_str = f"self.{name} = {name}"
 
+        ann_str, default_str = None, None
         if annotation is not None:
-            param_str = f"{name}: {annotation.as_string()}"
-        else:
-            param_str = name
+            ann_str = annotation.as_string()
 
         if value:
-            if isinstance(value, nodes.Call) and _looks_like_dataclass_field_call(
-                value, check_scope=False
-            ):
-                result = _get_field_default(value)
+            if is_field:
+                result = _get_field_default(value)  # type: ignore[arg-type]
                 if result:
                     default_type, default_node = result
                     if default_type == "default":
-                        param_str += f" = {default_node.as_string()}"
+                        default_str = default_node.as_string()
                     elif default_type == "default_factory":
-                        param_str += f" = {DEFAULT_FACTORY}"
+                        default_str = DEFAULT_FACTORY
                         assignment_str = (
                             f"self.{name} = {default_node.as_string()} "
                             f"if {name} is {DEFAULT_FACTORY} else {name}"
                         )
             else:
-                param_str += f" = {value.as_string()}"
+                default_str = value.as_string()
+        elif property_node:
+            # We set the result of the property call as default
+            # This hides the fact that this would normally be a 'property object'
+            # But we can't represent those as string
+            try:
+                # Call str to make sure also Uninferable gets stringified
+                default_str = str(
+                    next(property_node.infer_call_result(None)).as_string()
+                )
+            except (InferenceError, StopIteration):
+                pass
+        else:
+            # Even with `init=False` the default value still can be propogated to
+            # later assignments. Creating weird signatures like:
+            # (self, a: str = 1) -> None
+            previous_default = _get_previous_field_default(node, name)
+            if previous_default:
+                default_str = previous_default.as_string()
+
+        # Construct the param string to add to the init if necessary
+        param_str = name
+        if ann_str is not None:
+            param_str += f": {ann_str}"
+        if default_str is not None:
+            param_str += f" = {default_str}"
+
+        # If the field is a kw_only field, we need to add it to the kw_only_params
+        # This overwrites whether or not the class is kw_only decorated
+        if is_field:
+            kw_only = [k for k in value.keywords if k.arg == "kw_only"]  # type: ignore[union-attr]
+            if kw_only:
+                if kw_only[0].value.bool_value():
+                    kw_only_params.append(param_str)
+                else:
+                    params.append(param_str)
+                continue
+        # If kw_only decorated, we need to add all parameters to the kw_only_params
+        if kw_only_decorated:
+            if name in prev_kw_only_store:
+                prev_kw_only_store[name] = (ann_str, default_str)
+            else:
+                kw_only_params.append(param_str)
+        else:
+            # If the name was previously seen, overwrite that data
+            # pylint: disable-next=else-if-used
+            if name in prev_pos_only_store:
+                prev_pos_only_store[name] = (ann_str, default_str)
+            elif name in prev_kw_only_store:
+                params = [name, *params]
+                prev_kw_only_store.pop(name)
+            else:
+                params.append(param_str)
 
-        params.append(param_str)
         if not init_var:
             assignments.append(assignment_str)
 
-    prev_pos_only, prev_kw_only = _find_arguments_from_base_classes(
-        node, set(assign_names + ["self"])
+    prev_pos_only, prev_kw_only = _parse_arguments_into_strings(
+        prev_pos_only_store, prev_kw_only_store
     )
 
     # Construct the new init method paramter string
-    params_string = "self, "
-    if prev_pos_only:
-        params_string += prev_pos_only
-    if not kw_only_decorated:
-        params_string += ", ".join(params)
-
+    # First we do the positional only parameters, making sure to add the
+    # the self parameter and the comma to allow adding keyword only parameters
+    params_string = "" if "self" in prev_pos_only else "self, "
+    params_string += prev_pos_only + ", ".join(params)
     if not params_string.endswith(", "):
         params_string += ", "
 
-    if prev_kw_only:
-        params_string += "*, " + prev_kw_only
-        if kw_only_decorated:
-            params_string += ", ".join(params) + ", "
-    elif kw_only_decorated:
-        params_string += "*, " + ", ".join(params) + ", "
+    # Then we add the keyword only parameters
+    if prev_kw_only or kw_only_params:
+        params_string += "*, "
+    params_string += f"{prev_kw_only}{', '.join(kw_only_params)}"
 
     assignments_string = "\n    ".join(assignments) if assignments else "pass"
     return f"def __init__({params_string}) -> None:\n    {assignments_string}"
@@ -364,7 +442,7 @@ def _looks_like_dataclass_decorator(
     except (InferenceError, StopIteration):
         inferred = Uninferable
 
-    if inferred is Uninferable:
+    if isinstance(inferred, UninferableBase):
         if isinstance(node, nodes.Name):
             return node.name in decorator_names
         if isinstance(node, nodes.Attribute):
@@ -404,7 +482,7 @@ def _looks_like_dataclass_field_call(
     If check_scope is False, skips checking the statement and body.
     """
     if check_scope:
-        stmt = node.statement(future=True)
+        stmt = node.statement()
         scope = stmt.scope()
         if not (
             isinstance(stmt, nodes.AnnAssign)
@@ -425,8 +503,20 @@ def _looks_like_dataclass_field_call(
     return inferred.name == FIELD_NAME and inferred.root().name in DATACLASS_MODULES
 
 
+def _looks_like_dataclasses(node: nodes.Module) -> bool:
+    return node.qname() == "dataclasses"
+
+
+def _resolve_private_replace_to_public(node: nodes.Module) -> None:
+    """In python/cpython@6f3c138, a _replace() method was extracted from
+    replace(), and this indirection made replace() uninferable."""
+    if "_replace" in node.locals:
+        node.locals["replace"] = node.locals["_replace"]
+
+
 def _get_field_default(field_call: nodes.Call) -> _FieldDefaultReturn:
-    """Return a the default value of a field call, and the corresponding keyword argument name.
+    """Return a the default value of a field call, and the corresponding keyword
+    argument name.
 
     field(default=...) results in the ... node
     field(default_factory=...) results in a Call node with func ... and no arguments
@@ -449,8 +539,10 @@ def _get_field_default(field_call: nodes.Call) -> _FieldDefaultReturn:
             lineno=field_call.lineno,
             col_offset=field_call.col_offset,
             parent=field_call.parent,
+            end_lineno=field_call.end_lineno,
+            end_col_offset=field_call.end_col_offset,
         )
-        new_call.postinit(func=default_factory)
+        new_call.postinit(func=default_factory, args=[], keywords=[])
         return "default_factory", new_call
 
     return None
@@ -458,29 +550,19 @@ def _get_field_default(field_call: nodes.Call) -> _FieldDefaultReturn:
 
 def _is_class_var(node: nodes.NodeNG) -> bool:
     """Return True if node is a ClassVar, with or without subscripting."""
-    if PY39_PLUS:
-        try:
-            inferred = next(node.infer())
-        except (InferenceError, StopIteration):
-            return False
+    try:
+        inferred = next(node.infer())
+    except (InferenceError, StopIteration):
+        return False
 
-        return getattr(inferred, "name", "") == "ClassVar"
-
-    # Before Python 3.9, inference returns typing._SpecialForm instead of ClassVar.
-    # Our backup is to inspect the node's structure.
-    return isinstance(node, nodes.Subscript) and (
-        isinstance(node.value, nodes.Name)
-        and node.value.name == "ClassVar"
-        or isinstance(node.value, nodes.Attribute)
-        and node.value.attrname == "ClassVar"
-    )
+    return getattr(inferred, "name", "") == "ClassVar"
 
 
 def _is_keyword_only_sentinel(node: nodes.NodeNG) -> bool:
     """Return True if node is the KW_ONLY sentinel."""
     if not PY310_PLUS:
         return False
-    inferred = helpers.safe_infer(node)
+    inferred = safe_infer(node)
     return (
         isinstance(inferred, bases.Instance)
         and inferred.qname() == "dataclasses._KW_ONLY_TYPE"
@@ -511,7 +593,7 @@ _INFERABLE_TYPING_TYPES = frozenset(
 
 def _infer_instance_from_annotation(
     node: nodes.NodeNG, ctx: context.InferenceContext | None = None
-) -> Iterator[type[Uninferable] | bases.Instance]:
+) -> Iterator[UninferableBase | bases.Instance]:
     """Infer an instance corresponding to the type annotation represented by node.
 
     Currently has limited support for the typing module.
@@ -536,18 +618,26 @@ def _infer_instance_from_annotation(
         yield klass.instantiate_class()
 
 
-AstroidManager().register_transform(
-    nodes.ClassDef, dataclass_transform, is_decorated_with_dataclass
-)
+def register(manager: AstroidManager) -> None:
+    if PY313_PLUS:
+        manager.register_transform(
+            nodes.Module,
+            _resolve_private_replace_to_public,
+            _looks_like_dataclasses,
+        )
 
-AstroidManager().register_transform(
-    nodes.Call,
-    inference_tip(infer_dataclass_field_call, raise_on_overwrite=True),
-    _looks_like_dataclass_field_call,
-)
+    manager.register_transform(
+        nodes.ClassDef, dataclass_transform, is_decorated_with_dataclass
+    )
 
-AstroidManager().register_transform(
-    nodes.Unknown,
-    inference_tip(infer_dataclass_attribute, raise_on_overwrite=True),
-    _looks_like_dataclass_attribute,
-)
+    manager.register_transform(
+        nodes.Call,
+        inference_tip(infer_dataclass_field_call, raise_on_overwrite=True),
+        _looks_like_dataclass_field_call,
+    )
+
+    manager.register_transform(
+        nodes.Unknown,
+        inference_tip(infer_dataclass_attribute, raise_on_overwrite=True),
+        _looks_like_dataclass_attribute,
+    )
